@@ -1,6 +1,6 @@
 ## -------------------------------------------------------------------------------------------------
 ## -- Project : MLPro - A Synoptic Framework for Standardized Machine Learning Tasks
-## -- Package : mlpro
+## -- Package : mlpro.rl
 ## -- Module  : howto_rl_008_wrap_mlpro_environment_to_gym_environment.py
 ## -------------------------------------------------------------------------------------------------
 ## -- History :
@@ -11,10 +11,11 @@
 ## -- 2021-12-22  1.0.2     DA       Cleaned up a bit
 ## -- 2022-03-21  1.0.3     MRD      Use Gym Env Checker
 ## -- 2022-05-30  1.0.4     DA       Little refactoring
+## -- 2022-07-28  1.0.5     SY       Update due to the latest introduction of Gym 0.25
 ## -------------------------------------------------------------------------------------------------
 
 """
-Ver. 1.0.4 (2022-05-30)
+Ver. 1.0.5 (2022-07-28)
 
 This module shows how to wrap a native MLPro environment class to OpenAI Gym environment.
 """
@@ -28,5 +29,5 @@ from gym.utils.env_checker import check_env
 
 
 mlpro_env   = GridWorld(p_logging=Log.C_LOG_ALL)
-env         = WrEnvMLPro2GYM(mlpro_env, p_state_space=None, p_action_space=None)
+env         = WrEnvMLPro2GYM(mlpro_env, p_state_space=None, p_action_space=None, p_new_step_api=True)
 check_env(env)
