@@ -50,17 +50,17 @@ class UR5LabRealEnv(robot_real_env.RobotRealEnv):
         # We launch the ROSlaunch that spawns the robot into the world
         visualize = rospy.get_param("visualize", False)
         sim_mode = rospy.get_param("sim", True)
+        start_ur_driver = rospy.get_param("start_driver", True)
         robot_ip = rospy.get_param("robot_ip", "")
         reverse_ip = rospy.get_param("reverse_ip", "")
         reverse_port = rospy.get_param("reverse_port", 50001)
 
-        if not sim_mode:
+        if not sim_mode and start_ur_driver:
             assert robot_ip != "", "Please set the Robot IP"
 
-        # Start the ur_robot_driver
-        ROSLauncher(rospackage_name="ur_gazebo",
+        self.launch = ROSLauncher(rospackage_name="ur_gazebo",
                     launch_file_name="ur5_lab.launch",
-                    launch_arguments=dict(gui=visualize, sim=sim_mode, robot_ip=robot_ip, 
+                    launch_arguments=dict(gui=visualize, start_ur_driver=start_ur_driver, sim=sim_mode, robot_ip=robot_ip, 
                     reverse_ip=reverse_ip, reverse_port=reverse_port),
                     ros_ws_abspath=ros_ws_abspath)
         
@@ -428,6 +428,7 @@ class UR5LabSimEnv(robot_gazebo_env.RobotGazeboEnv):
         # We launch the ROSlaunch that spawns the robot into the world
         visualize = rospy.get_param("visualize", False)
         sim_mode = rospy.get_param("sim", True)
+        start_gazebo = rospy.get_param("start_gazebo", True)
         robot_ip = rospy.get_param("robot_ip", "")
         reverse_ip = rospy.get_param("reverse_ip", "")
         reverse_port = rospy.get_param("reverse_port", 50001)
@@ -437,7 +438,7 @@ class UR5LabSimEnv(robot_gazebo_env.RobotGazeboEnv):
 
         self.launch = ROSLauncher(rospackage_name="ur_gazebo",
                     launch_file_name="ur5_lab.launch",
-                    launch_arguments=dict(gui=visualize, sim=sim_mode, robot_ip=robot_ip, 
+                    launch_arguments=dict(gui=visualize, start_gazebo=start_gazebo, sim=sim_mode, robot_ip=robot_ip, 
                     reverse_ip=reverse_ip, reverse_port=reverse_port),
                     ros_ws_abspath=ros_ws_abspath)
         
@@ -456,7 +457,8 @@ class UR5LabSimEnv(robot_gazebo_env.RobotGazeboEnv):
 
         self._gripper = rospy.Publisher('/gripper_controller/command', String, 
                                             queue_size=1)
-                                            
+
+        rospy.sleep(5)      
         rospy.logdebug("UR5LabEnv unpausing")
         self.gazebo.unpauseSim()
                 
@@ -464,6 +466,7 @@ class UR5LabSimEnv(robot_gazebo_env.RobotGazeboEnv):
         
         rospy.Subscriber("/joint_states", JointState, self._joints_state_callback)
         rospy.Subscriber("/move_group/status", GoalStatusArray, self._plan_status_feedback)
+        self._tcp_pose_pub = rospy.Publisher("/tcp_pose", Pose, queue_size=10)
         self.last_contact_r = 0
         self.last_contact_l = 0
         
